@@ -9,87 +9,39 @@ from __future__ import unicode_literals
 from __future__ import absolute_import
 
 from .account import Account
-from ..utils import (check_strict,
-                     raiseCsb43Exception,
-                     DECIMAL,
-                     IS_PY3,
-                     m3_next,
-                     m3_unicode,
-                     m3_is_string,
-                     m3_enc)
+from ..utils.compat import (
+    m3_next,
+    m3_bytes,
+    m3_is_string,
+    m3_enc,
+)
+from ..utils import (
+    check_strict,
+    b_left_pad,
+    b_right_pad,
+)
 from ..utils import messages as msg
 from ..i18n import tr as _
+from .record import (
+    RecordSequence,
+    NextMixin,
+)
 import sys
 
 
-def showInfo(csb, fd=sys.stderr):
-    '''
-    :param csb: csb file object
-    :type csb: :class:`File`
+class FileContext(object):
 
-    :param fd: file descriptor
-    :type fd: :class:`file`
-    '''
-    print("*",
-          m3_enc((_("%d\taccount(s) read") % len(csb.accounts)), fd), file=fd)
-    print("*",
-          m3_enc((_("File properly closed:\t%s") % csb.is_closed()), fd),
-          file=fd)
-    print("*",
-          m3_enc((_("%d\trecord(s) read") % csb.abstract.totalRecords), fd),
-          file=fd)
-    for ac in csb.accounts:
-        print("*" * 60, file=fd)
-        print("* +",
-              m3_enc((_("Account:\t%s\t%s") % (ac.accountNumber,
-                                               ac.shortName)), fd),
-              file=fd)
-        print("*  ",
-              m3_enc((_(" From:\t%s") % ac.initialDate.strftime("%Y-%m-%d")),
-                     fd),
-              file=fd)
-        print("*  ",
-              m3_enc((_(" To:  \t%s") % ac.finalDate.strftime("%Y-%m-%d")),
-                     fd),
-              file=fd)
-        print("*  ", file=fd)
-        print("*  ",
-              m3_enc((_("%d\ttransaction(s) read") % len(ac.transactions)),
-                     fd),
-              file=fd)
-        print("*  ",
-              m3_enc((_("Account properly closed:\t%s") % ac.is_closed()), fd),
-              file=fd)
-        print("*  ", file=fd)
-        print("*  ",
-              m3_enc((_("Previous amount:\t%14.2f\t%s") %
-                      (ac.initialBalance, ac.currency.letter)), fd),
-              file=fd)
-        print("*  ",
-              m3_enc((_(" Income:        \t%14.2f\t%s") %
-                      (ac.abstract.income, ac.abstract.currency.letter)), fd),
-              file=fd)
-        print("*  ",
-              m3_enc((_(" Expense:       \t%14.2f\t%s") %
-                      (- ac.abstract.expense,
-                       ac.abstract.currency.letter)), fd),
-              file=fd)
-        print("*  ",
-              m3_enc((_("Balance:        \t%14.2f\t%s") %
-                      (ac.abstract.balance,
-                       ac.abstract.currency.letter)), fd),
-              file=fd)
-        print("*" * 60, file=fd)
+    def __init__(self):
+        self.line = None
 
 
-class File(object):
+class File(RecordSequence):
     '''
     A CSB43 file
 
     - Create a :class:`File` object from a file descriptor::
 
-        >>> from csb43.csb43 import File
-        >>> with open("csb_file.csb") as fd:
+        >>> with open("csb_file.csb") as fd: # doctest: +SKIP
         ...     f = File(fd)
         ...     # do something with f
         ...
@@ -102,63 +54,57 @@ class File(object):
 
     def __init__(self,
                  fd=None,
-                 strict=True,
-                 decimal=DECIMAL,
-                 yearFirst=True):
+                 **record_settings):
         '''
         :param fd: a csb file
         :type fd: :class:`file`
 
-        :param strict: treat warnings as exceptions when `True`
-        :type strict: :class:`bool`
-
-        :param decimal: number of digits to be considered as the decimal part \
-        in money
-        :type decimal: :class:`int`
-
-        :param yearFirst: switch between YYMMD [`True`] and DDMMYY [`False`] \
-        date formats
-        :type yearFirst: :class:`bool`
+        :param \*\*record_settings: see :class:`csb43.csb43.RecordSequence`
 
         :raises: :class:`csb43.utils.Csb43Exception`
         '''
 
+        super(File, self).__init__(**record_settings)
         self.__accounts = []
-        self.__strict = strict
         self.__closing = None
-        self.__decimal = decimal
-        self.__yearFirst = yearFirst
         self.__numRecords = 0
-        self.__fromFile = False
+        self._file_context = None
 
         if fd is not None:
 
-            self.__fromFile = True
+            self._file_context = FileContext()
 
             def skip():
                 pass
 
-            launcher = {'00': skip,
-                        '11': self.add_account,
-                        '22': self.add_transaction,
-                        '23': self.add_item,
-                        '24': self.add_exchange,
-                        '33': self.close_account,
-                        '88': self.close_file}
+            launcher = {b'00': skip,
+                        b'11': self.add_account,
+                        b'22': self.add_transaction,
+                        b'23': self.add_item,
+                        b'24': self.add_exchange,
+                        b'33': self.close_account,
+                        b'88': self.close_file}
 
-            for line in fd:
-                line = line.rstrip('\n\r')
+            for idx, line in enumerate(fd):
+                self._file_context.line = idx
+                line = self.str_encode(line).rstrip(b'\n\r')
                 launcher.get(line[0:2], self.__unknownRecord)(line)
                 self.__numRecords += 1
 
-            self.__fromFile = False
+            self._file_context = None
         #else:
             #pass
 
-    def __unknownRecord(self, line=''):
-        raiseCsb43Exception(msg.BAD_RECORD(line), self.__strict)
+    def __unknownRecord(self, line=b''):
+        self._raise_error(msg.BAD_RECORD(line))
 
-    def _get_accounts(self):
+    # accounts
+    #
+    # repr: [Account]
+
+    @property
+    def accounts(self):
+        ":rtype: :class:`list` of accounts"
         return self.__accounts
 
     def get_last_account(self):
@@ -179,10 +125,12 @@ class File(object):
         if isinstance(record, Account):
             self.__accounts.append(record)
         else:
-            self.__accounts.append(Account(record,
-                                           self.__strict,
-                                           decimal=self.__decimal,
-                                           yearFirst=self.__yearFirst))
+            self.__accounts.append(
+                Account(
+                    record,
+                    **self._settings()
+                )
+            )
 
     def add_transaction(self, record):
         '''
@@ -261,7 +209,7 @@ class File(object):
 
         If record is `None`, a new abstract is generated::
 
-            >>> c = csb.File()
+            >>> c = File()
             >>> c.is_closed()
             False
             >>> c.close_file()
@@ -269,7 +217,7 @@ class File(object):
             True
             >>> c.abstract.totalRecords
             0
-            >>> c.add_account(csb.Account())
+            >>> c.add_account(Account())
             >>> c.abstract.totalRecords
             0
             >>> c.close_file()
@@ -281,9 +229,10 @@ class File(object):
         If record is not empty, the number of records of `File` must be
         coincident with the quantity given in `record`::
 
-            >>> cf = csb.ClosingFile()
+            >>> c = File()
+            >>> cf = ClosingFile()
             >>> cf.totalRecords = 5
-            >>> c.close_file(cf)
+            >>> c.close_file(cf) # doctest: +SKIP
             Traceback (most recent call last):
             File "<stdin>", line 1, in <module>
             File "csb43/csb43/csb_file.py", line 200, in close_file
@@ -294,29 +243,30 @@ class File(object):
 incongruente: total de registros 5 != 1
             >>>
         '''
-        if self.__fromFile and (self.__closing is not None):
-            raiseCsb43Exception(_("trying to close an already closed file"),
-                                self.__strict)
+        settings = self._settings()
+
+        if self._file_context and (self.__closing is not None):
+            self._raise_error(_("trying to close an already closed file"))
 
         if record is not None:
             if isinstance(record, ClosingFile):
                 self.__closing = record
             else:
-                self.__closing = ClosingFile(record, self.__strict)
+                self.__closing = ClosingFile(record, **settings)
 
             n_r1 = int(self.__closing.totalRecords)
             n_r2 = self._get_num_records()
             if n_r1 != n_r2:
-                raiseCsb43Exception(
+                self._raise_error(
                     _('incongruent closing record of file: '
-                      'total records %d != %d') % (n_r1, n_r2),
-                    self.__strict)
+                      'total records %d != %d') % (n_r1, n_r2)
+                )
         else:
-            self.__closing = ClosingFile(strict=self.__strict)
+            self.__closing = ClosingFile(**settings)
             self.__closing.totalRecords = self._get_num_records()
 
     def _get_num_records(self):
-        if self.__fromFile:
+        if self._file_context:
             return self.__numRecords
         else:
             return sum([len([x for x in ac]) for ac in self.accounts])
@@ -327,11 +277,16 @@ incongruente: total de registros 5 != 1
         '''
         return self.__closing is not None
 
-    def _get_closing(self):
+    @property
+    def abstract(self):
+        ":rtype: :class:`ClosingFile` file abstract"
         return self.__closing
 
-    def as_dict(self):
+    def as_dict(self, decimal_fallback=None):
         '''
+        :param decimal_fallback: decimal number fallback representation
+        :type record: :class:`bool`
+
         :rtype: a representation of this object as a :class:`dict`. The keys \
         will be localised
 
@@ -340,7 +295,7 @@ incongruente: total de registros 5 != 1
         >>> f.add_account(csb.Account())
         >>> f.add_transaction(csb.Transaction())
         >>> import pprint
-        >>> pprint.pprint(f.as_dict())
+        >>> pprint.pprint(f.as_dict()) # doctest: +SKIP
         {u'cuentas': [{u'balance_inicial': None,
                     u'codigo_de_entidad': None,
                     u'codigo_de_sucursal': None,
@@ -361,7 +316,7 @@ incongruente: total de registros 5 != 1
                     u'numero_de_cuenta': None}]}
         '''
         return {
-            _("accounts"): [x.as_dict() for x in self.accounts]
+            _("accounts"): [x.as_dict(decimal_fallback=decimal_fallback) for x in self.accounts]
         }
 
     def __iter__(self):
@@ -373,19 +328,19 @@ incongruente: total de registros 5 != 1
         >>> f.add_account(csb.Account())
         >>> f.add_transaction(csb.Transaction())
         >>> for x in f:
-        ...     print x
+        ...     print(x)
         ...
-        11                  000000000000000000000000000000
-        22    0000000000000000000001000000000000000000000000000000000000
-        88999999999999999999000002
+        b'11                  000000000000000000000000000000                              '
+        b'22    0000000000000000000001000000000000000000000000000000000000                '
+        b'88999999999999999999000002                                                      '
         >>>
 
         '''
-        if not self.__fromFile:
+        if not self._file_context:
             self.close_file()
         return _FileIter(self)
 
-    def __str__(self):
+    def __bytes__(self):
         ''':rtype: representation of this object as `CSB43` records \
         (using `\\\\n` as separator)
 
@@ -393,22 +348,75 @@ incongruente: total de registros 5 != 1
         >>> f = csb.File()
         >>> f.add_account(csb.Account())
         >>> f.add_transaction(csb.Transaction())
-        >>> print f
+        >>> print(f) # doctest: +SKIP
         11                  000000000000000000000000000000
         22    0000000000000000000001000000000000000000000000000000000000
         88999999999999999999000002
         '''
-        return '\n'.join([x for x in self])
+        return b'\n'.join(x for x in self)
 
-    #**** Properties ****
+    def showInfo(self, fd=sys.stderr):
+        '''
+        :param fd: file descriptor where to write to
+        :type fd: :class:`file`
+        '''
 
-    accounts = property(_get_accounts, None, None,
-                        ":rtype: :class:`list` of accounts")
-    abstract = property(_get_closing, None, None,
-                        ":rtype: :class:`ClosingFile` file abstract")
+        header = (
+            _("Encoding: {encoding:s}"),
+            _("{naccounts:d}\taccount(s) read"),
+            _("File properly closed:\t{file_is_closed}"),
+            _("{total_records:d}\trecord(s) read"),
+        )
+
+        txt = "\n".join("* %s" % x for x in header).format(
+            encoding=self.str_encoding,
+            naccounts=len(self.accounts),
+            file_is_closed=self.is_closed(),
+            total_records=self.abstract.totalRecords,
+        )
+
+        item_ac = (
+            _("+ Account:\t{account_number}\t{short_name}"),
+            _("   From:\t{date_from}"),
+            _("   To:  \t{date_to}"),
+            "",
+            _("{ntransactions:d} transaction(s) read"),
+            _("Account properly closed:\t{ac_is_closed}"),
+            _("Information mode:\t{information_mode}"),
+            _("Previous amount:\t{ib_amount:14.2f}\t{ib_currency}"),
+            _(" Income:        \t{inc_amount:14.2f}\t{inc_currency}"),
+            _(" Expense:       \t{exp_amount:14.2f}\t{exp_currency}"),
+            _("Balance:        \t{bal_amount:14.2f}\t{bal_currency}"),
+        )
+
+        template = "\n".join("* %s" % x for x in item_ac)
+
+        print(m3_enc(txt, fd), file=fd)
+
+        for ac in self.accounts:
+            txt = template.format(
+                account_number=ac.accountNumber,
+                short_name=ac.shortName,
+                date_from=ac.initialDate.strftime("%Y-%m-%d"),
+                date_to=ac.finalDate.strftime("%Y-%m-%d"),
+                ntransactions=len(ac.transactions),
+                ac_is_closed=ac.is_closed(),
+                information_mode=ac.informationMode,
+                ib_amount=ac.initialBalance,
+                ib_currency=ac.currency.alpha_3,
+                inc_amount=ac.abstract.income,
+                inc_currency=ac.abstract.currency.alpha_3,
+                exp_amount=ac.abstract.expense,
+                exp_currency=ac.abstract.currency.alpha_3,
+                bal_amount=ac.abstract.balance,
+                bal_currency=ac.abstract.currency.alpha_3,
+            )
+            print("*" * 60, file=fd)
+            print(txt, file=fd)
+            print("*" * 60, file=fd)
 
 
-class _FileIter(object):
+class _FileIter(NextMixin):
 
     def __init__(self, f):
         self.__output = []
@@ -419,7 +427,7 @@ class _FileIter(object):
         self.__iter = iter(self.__output)
         self.__acc = None
 
-    def next(self):
+    def __next__(self):
         if self.__acc:
             try:
                 return m3_next(self.__acc)
@@ -429,22 +437,19 @@ class _FileIter(object):
 
         if isinstance(now, Account):
             self.__acc = iter(now)
-            return self.next()
+            return self.__next__()
         else:
-            return str(now)
-
-    if IS_PY3:
-        def __next__(self):
-            return self.next()
+            return m3_bytes(now)
 
 
-class ClosingFile(object):
+class ClosingFile(RecordSequence):
     '''
     A File abstract, given by a termination record
 
     Create a :class:`ClosingFile` object from a `CSB43` string record::
 
-        >>> from csb43.csb43 import ClosingFile
+        >>> record = '8899999999999999999900012\
+3                                                      '
         >>> c = ClosingFile(record)
 
     From an empty object to a `CSB43` string record::
@@ -457,7 +462,8 @@ class ClosingFile(object):
 
     '''
 
-    def __init__(self, record=None, strict=True):
+    def __init__(self, record=None,
+                 **record_settings):
         '''
         :param record: csb record
         :type record: :class:`basestring` or `None`
@@ -467,73 +473,86 @@ class ClosingFile(object):
 
         :raises: :class:`csb43.utils.Csb43Exception`
         '''
-        self.__strict = strict
+        super(ClosingFile, self).__init__(**record_settings)
 
         self.__totalRecords = None
         self.__padding = None
 
         if record is not None:
+            chk_args = self._get_check_args()
+
+            record = self.str_encode(record)
             if not ClosingFile.is_valid(record):
-                raiseCsb43Exception(msg.BAD_RECORD(record), self.__strict)
+                self._raise_error(msg.BAD_RECORD(record))
 
-            self._check_nines(record[2:20], self.__strict)
-            self._set_total_records(record[20:26], self.__strict)
-            self._set_padding(record[26:80], self.__strict)
-
-    def _get_total_records(self):
-        if self.__totalRecords is None:
-            return None
-        return int(self.__totalRecords)
-
-    def _get_external_total_records(self):
-        if self.__totalRecords is None:
-            return None
-        return int(self.__totalRecords)
-
-    def _get_padding(self):
-        return self.__padding
-
-    @check_strict(r'^\d{6}$')
-    def _set_total_records(self, value, strict=True):
-        self.__totalRecords = value
-
-    def _set_external_total_records(self, value):
-        self._set_total_records(m3_unicode(value).rjust(6, '0'))
-
-    @check_strict(r'^.{0,54}$')
-    def _set_padding(self, value, strict=True):
-        self.__padding = value
-
-    @check_strict(r'^9{18}$')
-    def _check_nines(self, value, strict=True):
-        pass
+            self._check_nines(record[2:20], **chk_args)
+            self._set_total_records(record[20:26], **chk_args)
+            self._set_padding(record[26:80], **chk_args)
 
     @staticmethod
     def is_valid(record):
         return m3_is_string(record)\
-            and (27 <= len(record) <= 80) and (record[0:2] == '88')
+            and (27 <= len(record) <= 80) and (record[0:2] == b'88')
 
-    def __str__(self):
-        ''':rtype: representation of this object as `CSB43` records \
-        (using `\\\\n` as separator)'''
-        return "88{nines}{records:0>6}{padding: <54}".format(
-            nines='9' * 18,
-            records=self._get_total_records() or '',
-            padding=self._get_padding() or ''
-        )
+    # padding
+    #
+    # repr: bytes
+    # user in: text
+    # user out: bytes
 
-    #**** Properties ****
+    @check_strict(br'^.{0,54}$')
+    def _set_padding(self, value, **chk_args):
+        self.__padding = value.rstrip(b" ")
 
-    totalRecords = property(_get_external_total_records,
-                            _set_external_total_records, None,
-                            """total number of entries
+    @property
+    def padding(self):
+        """padding"""
+        return self.__padding
 
+    @padding.setter
+    def padding(self, value):
+        value = self.str_encode(value)
+        self._set_padding(value)
+
+    # total records
+    #
+    # repr: bytes
+    # user in: text, number
+    # user out: int
+
+    @check_strict(br'^\d{6}$')
+    def _set_total_records(self, value, **chk_args):
+        self.__totalRecords = value
+
+    def _get_total_records(self):
+        return self.__totalRecords
+
+    @property
+    def totalRecords(self):
+        """total number of entries
+
+>>> c = ClosingFile()
 >>> c.totalRecords = 34
 >>> c.totalRecords
 34
->>> c.totalRecords = '115'
->>> c.totalRecords
-115
+"""
+        if self.__totalRecords is None:
+            return None
+        return int(self.__totalRecords)
 
-""")
-    padding = property(_get_padding, _set_padding, None, "padding")
+    @totalRecords.setter
+    def totalRecords(self, value):
+        field = self._enc_left_pad(value, n=6, fill=b"0")
+        self._set_total_records(field)
+
+    @check_strict(br'^9{18}$')
+    def _check_nines(self, value, **chk_args):
+        pass
+
+    def __bytes__(self):
+        ''':rtype: representation of this object as `CSB43` records \
+        (using `\\\\n` as separator)'''
+        return (b"88" +
+                b"9" * 18 +
+                b_left_pad(self._get_total_records() or b'', 6, b"0") +
+                b_right_pad(self.padding or b'', 54))
