@@ -3,8 +3,9 @@
 '''
 from __future__ import unicode_literals
 from __future__ import absolute_import
+from __future__ import print_function
 
-from .transaction import Transaction
+from . import Transaction
 
 
 #: conversion table CSB - Homebank for pay modes
@@ -24,13 +25,13 @@ def convertFromCsb(csb):
     :rtype: :class:`list` of :class:`Transaction`
 
     >>> # Homebank
-    >>> from csb43 import csb_43, homebank
+    >>> from csb43 import csb43
     >>> #
-    >>> csbFile = csb_43.File(open("movimientos.csb"), strict=False)
+    >>> csbFile = csb43.File(open("movimientos.csb"), strict=False) # doctest: +SKIP
     >>> #
     >>> # print to stdout
-    >>> for line in homebank.convertFromCsb(csbFile):
-    ...    print line
+    >>> for line in convertFromCsb(csbFile): # doctest: +SKIP
+    ...    print(line)
     ...
     '''
 
@@ -45,8 +46,10 @@ def convertFromCsb(csb):
             record.info = t.ownItem
             record.payee = t.reference1.rstrip(' ')
 
-            info = ["%s: %s" % (x.item1.rstrip(' '), x.item2.rstrip(' '))
-                    for x in t.optionalItems]
+            info = [
+                "%s: %s" % (x.item1.rstrip(' '), x.item2.rstrip(' '))
+                for x in t.optionalItems
+            ]
             record.description = "/".join(info)
             record.amount = "%1.2f" % t.amount
 
