@@ -11,7 +11,7 @@ import pkg_resources
 
 import sys
 
-IS_PY3 = int(sys.version[0:1]) >= 3
+IS_PY2 = int(sys.version[0:1]) < 3
 
 # Change this variable to your app name!
 #  The translation files will be under
@@ -23,25 +23,31 @@ LOCALE_DIR = pkg_resources.resource_filename(__name__, '')
 DEFAULT_LANGUAGES = os.environ.get('LANG', '').split(':')
 DEFAULT_LANGUAGES += ['en']
 
-lc, encoding = locale.getdefaultlocale()
+languages = []
+try:
+    lc, encoding = locale.getdefaultlocale()
+except ValueError:
+    lc, encoding = None, None
 if lc:
-    languages = [lc]
+    languages.append(lc)
 
 languages += DEFAULT_LANGUAGES
 mo_location = LOCALE_DIR
 
-if IS_PY3:
-    gettext.install(True, localedir=None)
-else:
+if IS_PY2:
     gettext.install(True, localedir=None, unicode=1)
+else:
+    gettext.install(True, localedir=None)
 
 gettext.textdomain(DOMAIN_NAME)
 
 gettext.bind_textdomain_codeset(DOMAIN_NAME, "UTF-8")
 
-language = gettext.translation(DOMAIN_NAME,
-                               mo_location,
-                               languages=languages,
-                               fallback=True)
+language = gettext.translation(
+    DOMAIN_NAME,
+    mo_location,
+    languages=languages,
+    fallback=True
+)
 
-tr = language.ugettext if not IS_PY3 else language.gettext
+tr = language.ugettext if IS_PY2 else language.gettext
