@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-CURRENCY_DATA = [
+CURRENCY_DATA = (
     ('AED', '784'),
     ('AFN', '971'),
     ('ALL', '008'),
@@ -184,4 +184,4 @@ CURRENCY_DATA = [
     ('ZAR', '710'),
     ('ZMK', '894'),
     ('ZWD', '716'),
-]
+)
